/***************************************************************************
 * First Cut at Ports0 interface: 
 * Author: Neel Sundaresan, Indiana University
 ***************************************************************************/

#ifndef _LOCAL_INLINES_H
#define _LOCAL_INLINES_H

#define amILeader() (ll_MyProcessorNum() == 0)

#define myBarrierPtr() (ll_CurrentContextId() == -1 ? subdomainData.proc_barrier_p : ll_CurrentContext()->barrier_p)

#define setThread(t,rhs) ((t) = (rhs))
#define setThisThread(t) (setThread(ll_ThisThread(), t))

/* there can be multiple main threads per context (1 per processor) */
/* but only one root thread (the thread running on the 0th */
/* processor!). is this restriction required? how about delivering it */
/* to an arbitrary main thread depending on who picks up? */
#define mainThread(processor_num) (mainThreadOnContext(ll_CurrentContext(),(processor_num)))


#ifdef _SINGLE_PROCESSOR_PER_PC
#define mainThreadOnContext(context,pnum) ((context)->mainThread_ptr)
#else
#define mainThreadOnContext(context,pnum) \
           ((context)->mainThreadList[(pnum)])
#endif

#define MyWatchdogThread() (ll_CurrentContext()->watchdogThread)
#define WatchdogThread(on_pc) (ll_Context((on_pc))->watchdogThread)

#define MyMsgThread()     (subdomainData.msgThread)

#define registryLock() (ll_CurrentContext()->registry_lock)
#define registryList() (ll_CurrentContext()->registry_list)
#define setRegistryList(t) (ll_CurrentContext()->registry_list = (t))
#define registryIndex() (ll_CurrentContext()->registry_index)
#define incrRegistryIndex() (++ll_CurrentContext()->registry_index)
#define pdataKeyIndex() (subdomainData.neel_pdata_key_index)
#define incrPdataKeyIndex() (++subdomainData.neel_pdata_key_index)

#define readyQlock() (ll_CurrentContext()->readyQlock_ptr)
#define readyQhead() (ll_CurrentContext()->readyQhead_ptr)
#define readyQtail() (ll_CurrentContext()->readyQtail_ptr)
#define setReadyQhead(t) (ll_CurrentContext()->readyQhead_ptr = (t))
#define setReadyQtail(t) (ll_CurrentContext()->readyQtail_ptr = (t))

/* context switch(cs) related stuff */
#ifdef _SINGLE_PROCESSOR_PER_PC
#define cs_getState() (ll_CurrentContext()->threadState)
#define cs_setState(x) (ll_CurrentContext()->threadState = (x))
#define cs_getoldThread() (ll_CurrentContext()->oldThread)
#define cs_setoldThread(x) (ll_CurrentContext()->oldThread =(x))
#else
#define cs_getState() (ll_CurrentContext()->threadStates[ll_MyProcessorNum()])
#define cs_setState(x) (ll_CurrentContext()->threadStates[ll_MyProcessorNum()] = (x))
#define cs_getoldThread() (ll_CurrentContext()->oldThreads[ll_MyProcessorNum()])
#define cs_setoldThread(x) (ll_CurrentContext()->oldThreads[ll_MyProcessorNum()]=(x))
#endif

/* yielder related stuff */
#ifdef _SINGLE_PROCESSOR_PER_PC
#define ith_yielder(pid) (ll_CurrentContext()->yielder)
#define set_ith_yielder(pid, thr) (ll_CurrentContext()->yielder=(thr))
#else
#define ith_yielder(pid) (ll_CurrentContext()->yielder[(pid)])
#define set_ith_yielder(pid, thr) (ll_CurrentContext()->yielder[(pid)]=(thr))
#endif

/* communication related stuff */

#define theWaitersLock() (ll_CurrentContext()->theWaitersLock)
#define theWaitersLockP() (&ll_CurrentContext()->theWaitersLock)
#define setWaitersLock(x) (ll_CurrentContext()->theWaitersLock = (x))
#define theWaiters() (ll_CurrentContext()->theWaiters)
#define setWaiters(x) (ll_CurrentContext()->theWaiters = (x))
#define numPorts() (ll_CurrentContext()->numPorts)
#define setNumPorts(x) (ll_CurrentContext()->numPorts = (x))
#define numRecvWaiters() (ll_CurrentContext()->num_recv_waiters)
#define setNumRecvWaiters(x) (ll_CurrentContext()->num_recv_waiters = (x))
#define incrNumRecvWaiters() (ll_CurrentContext()->num_recv_waiters++)
#define decrNumRecvWaiters() (ll_CurrentContext()->num_recv_waiters--)
#define getFreeBufDescs() (ll_CurrentContext()->freeBufDescs)
#define freeBufDescsLock() (ll_CurrentContext()->freeBufDescsLock)
#define freeBufDescsLockP() (&ll_CurrentContext()->freeBufDescsLock)
#define setFreeBufDescs(x) (ll_CurrentContext()->freeBufDescs=(x))

/* communication stuff, mailbox.... 
   every context has a mail box into which other context threads can put
   messages into.. */
#define theMailBox() (ll_CurrentContext()->mailBox)
#define theMailBoxP() (&ll_CurrentContext()->mailBox)
#define IthMailBox(i) (ll_Context((i))->mailBox)
#endif





